/*
 * A region mapper for deployment in a new task.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <exec/common.h>
#include <exec/internal_pager.h>
#include <exec/stack.h>
#include <ipc/server.h>

#include "pager_object_server.h"



static InternalPager exec_pager;



int main(int argc, char *argv[])
{
  /* Initialise pager regions from region descriptions obtained via the
     auxiliary data. */

  l4re_aux_t *l4re_aux = exec_get_l4re_aux(argc, argv);

  printf("aux = {%s, %lx, %lx, %lx}\n", l4re_aux->binary, l4re_aux->kip_ds, l4re_aux->dbg_lvl, l4re_aux->ldr_flags);

  /* Skip past the auxiliary structure itself. */

  struct exec_region *region = (struct exec_region *) *((l4_addr_t *) (l4re_aux + 1));

  if (!region)
  {
    printf("Could not find regions.\n");
    return 1;
  }

  for (; region && (region->ds != L4_INVALID_CAP); region++)
  {
    printf("Adding region: {%lx, %lx, %lx, %lx}\n", region->start, region->size, region->flags, region->ds);
    exec_pager.add(*region);
  }

  /* Start the pager. */

  printf("Starting pager...\n");
  printf("Pager capability: %lx\n", l4re_env_get_cap("server"));
  printf("Main thread: %lx\n", l4re_env()->main_thread);

  ipc_server_loop_for(PagerObject, &exec_pager, "server");

  printf("Ending pager...\n");
  return 0;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
